#include <stdio.h>
#include <stdlib.h>
#include "boolean.h"
#include "order.h"
#include "readField.h"

OrderList emptyOrderList()
{
	return NULL;
}

Boolean isEmptyOrderList(OrderList list)
{
	return list == NULL;
}

OrderList addOrder(OrderList list, Order o)
{
	OrderList newHead = (OrderList)malloc(sizeof(OrderListElement));
	newHead->order = o;
	newHead->next = list;
	return newHead;
}

Order headOrder(OrderList list)
{
	return list->order;
}

OrderList orderListTail(OrderList list)
{
	return list->next;
}

void destroyOrderList(OrderList list)
{
	if (list != NULL)
	{
		destroyOrderList(list->next);
		free(list);
	}
}

Boolean readOrderFromTxt(FILE *f, Order* order, Component components[], int dim)
{
	int index;
	Boolean ok, continueRead;
	ok = fscanf(f, "%d;", &order->code);
	ok = ok && readField(f, order->addressee, ';');

	index = 0;
	continueRead = true;
	while (ok && continueRead)
	{
		int componentCode;
		ok = fscanf(f, "%d,%d", &componentCode, &order->lines[index].quantity) == 2;
		if (ok)
		{
			int componentIdx = findComponentByCode(components, dim, componentCode);
			ok = componentIdx >= 0;
			if (ok)
			{
				order->lines[index].comp = &components[componentIdx];
				index++;
				continueRead = fgetc(f) == ';';
			}
		}
	}
	order->linesCount = index;
	return ok;
}

OrderList readOrdersFromTxt(char fileName[], Component components[], int dim)
{
	Order order;
	OrderList list;
	FILE *f;

	f = fopen(fileName, "r");
	if (f == NULL)
	{
		return NULL;
	}

	list = emptyOrderList();
	while (readOrderFromTxt(f, &order, components, dim))
	{
		list = addOrder(list, order);
	}

	fclose(f);
	return list;
}

float getOrderLinePrice(OrderLine line)
{
	return line.quantity * line.comp->unitPrice;
}

float getOrderPrice(Order order)
{
	int i;
	float price = 0;
	for (i = 0; i < order.linesCount; i++)
	{
		price += getOrderLinePrice(order.lines[i]);
	}
	return price;
}

void printOrderList(OrderList list)
{
	while (!isEmptyOrderList(list))
	{
		int i;
		float orderPrice = 0;
		Order order = headOrder(list);
		printf("%d -- %s\n", order.code, order.addressee);
		for (i = 0; i < order.linesCount; i++)
		{
			float orderLinePrice = getOrderLinePrice(order.lines[i]);
			orderPrice += orderLinePrice;
			printf("\t%s x %d = %.2f\n", order.lines[i].comp->description, 
				order.lines[i].quantity, orderLinePrice);
		}
		printf("Order Total: %.2f\n\n", orderPrice);
		//printf("Order Total: %.2f\n\n", getOrderPrice(order)); //Meno ottimizzato ma pi chiaro e incapsulato
		list = orderListTail(list);
	}
}

void writeOrderPrices(char fileName[], OrderList list)
{
	FILE *f = fopen(fileName, "w");

	while (!isEmptyOrderList(list))
	{
		Order order = headOrder(list);
		fprintf(f, "%d,%.2f\n", order.code, getOrderPrice(order));
		list = orderListTail(list);
	}
}

void printUsedComponents(Component components[], int dim, OrderList list)
{
	int i;
	for (i = 0; i < dim; i++)
	{
		int count = 0;
		OrderList scanList = list;
		while (!isEmptyOrderList(scanList))
		{
			int j;
			Order order = headOrder(scanList);
			for (j = 0; j < order.linesCount; j++)
			{
				//if (order.lines[j].comp->code == components[i].code) oppure
				if (order.lines[j].comp == components + i)	//in OrderLine c' un ptr a Component (quello originale)
				{
					count++;
				}
			}
			scanList = orderListTail(scanList);
		}
		if (count != 0)
		{
			printf("code '%d' -- n. ordered: %d\n", components[i].code, count);
		}
	}
}
